/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "light.h"


Light::Light(int numColor, int status): mStatus(status) {	
	setNumColor(numColor);
}

Light::Light(int numColor, float coeff, int status): mStatus(status) {
	setNumColor(numColor, coeff);
}

Light::~Light() {
}

void Light::setNumColor(int numColor) {
	if(numColor == 0) {
		mColor = Color(14, 0., 1.);
		mAmbient = Color(0.05, 0.05, 0.05, 1.);
	} else {
		mColor = Color(numColor, 0.,1.);
		mAmbient = Color(13, 0.,1.);
	}	
}

void Light::setNumColor(int numColor, float coeff) {
	if(numColor == 0) {
		mColor = Color(14, coeff, 1.);
		mAmbient = Color(0.05, 0.05, 0.05, 1.);
	} else {
		mColor = Color(numColor, coeff,1.);
		mAmbient = Color(13, 0.,1.);
	}
}


void Light::activate(int num) {	
	lightId = GL_LIGHT0 + num;
	glEnable(lightId);
	glLightfv(lightId, GL_AMBIENT, mAmbient);
    glLightfv(lightId, GL_DIFFUSE, mColor);
    glLightfv(lightId, GL_SPECULAR, mColor);
}


void Light::setAmbient(Color ambient) {
	mAmbient = ambient;	
}


void Light::setColor(Color color) {
	mColor = color;
}

int Light::getLightId() {
	return lightId;
}


int Light::getStatus() {
	return mStatus;
}

void Light::switchOn() {
	if(mStatus!=2) {
		mStatus = 1;
	}
}

void Light::switchOff() {
	if(mStatus!=2) {
		mStatus = 0;
	}
}	



/************************************************************/
/*		LightPoint											*/
/************************************************************/

LightPoint::LightPoint(int numColor, int status, Vector3 position): Light(numColor, status) {
	mPosition = Vector4(position, 1.);
	setAttenuations(1., 0., 0.);
}

LightPoint::LightPoint(int numColor, int status, Vector3 position, float intensity, float d0) : Light(numColor, status) {
	mPosition = Vector4(position, 1.);
	setIntensityAndAttenuation (intensity, d0);
}

LightPoint::LightPoint(int numColor, float coeff, int status, Vector3 position, float intensity, float d0) : Light(numColor, coeff, status) {
	mPosition = Vector4(position, 1.);
	setIntensityAndAttenuation (intensity, d0);
}	

LightPoint::~LightPoint() {	
}


void LightPoint::activate(int num) {	
	Light::activate(num);	
	glLightfv(lightId, GL_POSITION, mPosition);
	glLightf(lightId, GL_CONSTANT_ATTENUATION, constantAttenuation);
	glLightf(lightId, GL_LINEAR_ATTENUATION, linearAttenuation);
	glLightf(lightId, GL_QUADRATIC_ATTENUATION, quadraticAttenuation);	
}

void LightPoint::release() {
	glLightfv(lightId, GL_POSITION, Vector4(0.,0.,1.,0.));
	glDisable(lightId);
}	

void LightPoint::setIntensityAndAttenuation(float intensity, float d0) {
	constantAttenuation = 1./intensity;
	linearAttenuation = 0.;
	quadraticAttenuation = 1./(intensity*d0*d0);
}

void LightPoint::setAttenuations(float constant, float linear, float quadratic) {
	constantAttenuation = constant;
	linearAttenuation = linear;
	quadraticAttenuation = quadratic;
}	
	
void LightPoint::setPosition(Vector3 position) {
	mPosition = Vector4(position, 1.);	
}


/************************************************************/
/*		LightDir										*/
/************************************************************/

LightDir::LightDir(int numColor, int status, Vector3 direction) : Light(numColor, status) {
	mDirection = Vector4(direction, 0.);
}
	
LightDir::LightDir(int numColor, float coeff, int status, Vector3 direction) : Light(numColor, coeff, status) {
	mDirection = Vector4(direction, 0.);
}
	
LightDir::~LightDir() {
}

void LightDir::activate(int num) {
	Light::activate(num);
	glLightfv(lightId, GL_POSITION, mDirection);
}	

void LightDir::release() {	
	glLightfv(lightId, GL_POSITION, Vector4(0.,0.,1.,0.));
	glDisable(lightId);
}	

void LightDir::setDirection(Vector3 direction) {
	mDirection = Vector4(direction, 0.);	
}



/************************************************************/
/*		LightSpot										*/
/************************************************************/

LightSpot::LightSpot(int numColor, int status, Vector3 position, Vector3 direction, float exponent, float cutoff): Light(numColor, status) {
	mPosition = Vector4(position, 1.);
	mDirection = direction;

	mExponent = exponent;
	mCutoff = cutoff;	
	
	setAttenuations(1., 0., 0.);	
}

LightSpot::LightSpot(int numColor, int status, Vector3 position, Vector3 direction, float exponent, float cutoff, float intensity, float d0):Light(numColor, status) {
	mPosition = Vector4(position, 1.);
	mDirection = direction;

	mExponent = exponent;
	mCutoff = cutoff;
	
	setIntensityAndAttenuation (intensity, d0);	
}	

LightSpot::LightSpot(int numColor, float coeff, int status, Vector3 position, Vector3 direction, float exponent, float cutoff, float intensity, float d0):Light(numColor, coeff, status) {
	mPosition = Vector4(position, 1.);
	mDirection = direction;

	mExponent = exponent;
	mCutoff = cutoff;
	
	setIntensityAndAttenuation (intensity, d0);	
}

LightSpot::~LightSpot() {
}


float LightSpot::getCutoff() {
	return mCutoff;
}

Vector3 LightSpot::getPosition() {
	return mPosition.getVector3();
}

Vector3 LightSpot::getDirection() {
	return mDirection;
}

void LightSpot::activate(int num) {
	Light::activate(num);	
	glLightfv(lightId, GL_POSITION, mPosition);
	glLightfv(lightId, GL_SPOT_DIRECTION, mDirection);
	glLightf(lightId, GL_CONSTANT_ATTENUATION, constantAttenuation);
	glLightf(lightId, GL_LINEAR_ATTENUATION, linearAttenuation);
	glLightf(lightId, GL_QUADRATIC_ATTENUATION, quadraticAttenuation);	
	glLightf(lightId, GL_SPOT_EXPONENT, mExponent);
	glLightf(lightId, GL_SPOT_CUTOFF, mCutoff);
}	

void LightSpot::release() {
	glLightfv(lightId, GL_POSITION, Vector4(0.,0.,1.,0.));
	glLightfv(lightId, GL_SPOT_DIRECTION, Vector3(0.,0.,-1.));
	glLightf(lightId, GL_SPOT_EXPONENT, 0.);
	glLightf(lightId, GL_SPOT_CUTOFF, 180.);
	glDisable(lightId);
}	


void LightSpot::setIntensityAndAttenuation(float intensity, float d0) {
	constantAttenuation = 1./intensity;
	linearAttenuation = 1./(intensity*2.*d0);
	quadraticAttenuation = 1./(intensity*2.*d0*d0);	
}

void LightSpot::setAttenuations(float constant, float linear, float quadratic) {
	constantAttenuation = constant;
	linearAttenuation = linear;
	quadraticAttenuation = quadratic;
}	

void LightSpot::setSpotDirection(Vector3 direction) {
	mDirection = direction;	
}

void LightSpot::setPosition(Vector3 position) {
	mPosition = Vector4(position, 1.);
}	


