/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "shader.h"

Shader::Shader(): id(0) {
}

Shader::Shader(std::vector<std::string> vsFilenames, std::vector<std::string> vsGeneratedSources, std::string vsMainSourceFile,
		   	   std::vector<std::string> fsFilenames, std::vector<std::string> fsGeneratedSources, std::string fsMainSourceFile) {
	id = loadProgram (vsFilenames, vsGeneratedSources, vsMainSourceFile, fsFilenames, fsGeneratedSources, fsMainSourceFile);			   
}
	
Shader::Shader (std::vector<std::string> vsFiles, std::vector<std::string> fsFiles) {
	id = loadProgram (vsFiles, fsFiles);
}
		
Shader::~Shader() {
	glDeleteProgram(id);	
}


GLuint Shader::getId() {
	return id;
}


GLuint Shader::loadProgram(GLuint vertexShader, GLuint fragmentShader) {
	GLuint prog = 0;
	GLint link_status = GL_TRUE;
	GLint logsize = 0;
	char *log = NULL;	
	
    // we create the program
	prog = glCreateProgram();
    
	// we attach the shaders	
	glAttachShader(prog, vertexShader);
	glAttachShader(prog, fragmentShader);
    
	glLinkProgram(prog);
    
	// we verify all is ok
	glGetProgramiv(prog, GL_LINK_STATUS, &link_status);
	if(link_status != GL_TRUE) {
		glGetProgramiv(prog, GL_INFO_LOG_LENGTH, &logsize);
		log = new char[logsize + 1];
		if(log == NULL) {
			glDeleteProgram(prog);
			glDeleteShader(vertexShader);
			glDeleteShader(fragmentShader);
            
			std::cerr << "cannot allocate memory!" << std::endl;
			return 0;
		}
		std::fill(log, log + logsize +1, '\0');
		glGetProgramInfoLog(prog, logsize, &logsize, log);
        
		std::cerr << "unable to link the program" << std::endl;
		std::cerr << log << std::endl;
        
		delete log;
		glDeleteProgram(prog);
        
		return 0;
	}
    
	return prog;
}


GLuint Shader::loadProgram(std::vector<std::string> vsFilenames, std::vector<std::string> vsGeneratedSources, std::string vsMainSourceFile,
								  std::vector<std::string> fsFilenames, std::vector<std::string> fsGeneratedSources, std::string fsMainSourceFile) {
	
	GLuint prog = 0;
	GLuint vs,fs;	
	
	ShaderLoader *vertexLoader = new ShaderLoader(GL_VERTEX_SHADER, vsFilenames, vsGeneratedSources, vsMainSourceFile);
	ShaderLoader *fragmentLoader = new ShaderLoader(GL_FRAGMENT_SHADER, fsFilenames,fsGeneratedSources, fsMainSourceFile);
									  
	vs = vertexLoader->getId();
	fs = fragmentLoader->getId();						  
									  
	if(vs == 0) {
		std::cout << "Vertex shader has not be loaded." << std::endl;
		return 0;
	}
	
	if(fs == 0) {
		std::cout << "Fragment shader has not be loaded." << std::endl;
		if(glIsShader(vs))
			glDeleteShader(vs);
		return 0;
	}
	
	prog = loadProgram(vs,fs);
									  
	delete vertexLoader;
	delete fragmentLoader;									  
    
	return prog;	
}


GLuint Shader::loadProgram(std::vector<std::string> vsFiles, std::vector<std::string> fsFiles) {
	GLuint prog = 0;
	GLuint vs, fs;
  
	ShaderLoader *vertexLoader = new ShaderLoader(GL_VERTEX_SHADER, vsFiles);
	ShaderLoader *fragmentLoader = new ShaderLoader(GL_FRAGMENT_SHADER, fsFiles);
		
	vs = vertexLoader->getId();
	if(vs == 0) {
		std::cout << "Vertex shader has not be loaded." << std::endl;		
		return 0;
	}
	
	fs = fragmentLoader->getId();
	if(fs == 0) {
		std::cout << "Fragment shader has not be loaded." << std::endl;		
		if(glIsShader(vs))
			glDeleteShader(vs);
		return 0;
	}
	
	prog = loadProgram(vs,fs);
	
    delete vertexLoader;
	delete fragmentLoader;	
	
	return prog;	
}
